' SFF Merge v1.33
' Utility to manage merging Updates with the main SFF library
'
' Valid Directives in directive file - tab separated
' B 0 message			Display message box. Use <BR> to put linefeed in message
' B 1 message			Display message box and terminate Use <BR> to put linefeed in message
' B 2 message			Display yes/no message box - terminate if no. Use <BR> to put linefeed in message
' D file				Delete file/folder - may be needed for eg _Tools
' F folder				Delete folder and contents - eg for series updates
' M file destdir		Move - move file to destdir instead of author dir.
' N 123.4				Require version 123.4 to run
' R						Indicates rollback - only for internal use - stops the generation of rollback files and info
' V 123.4				Make version 123.4 on completion
' Y SFFMerge Version	Check we're running an appropriate version.

'============= User modifiable ===============================================

BaseLibrary = ""    ' you can set this to the base of your library eg "N:\sfflibrary"
BaseUpdates = ""    ' you can set this to the base of your update path eg "N:\sffupdates"
DateSeparator = "/"
TimeSeparator = ":"

'=========== Don't change below here. ========================================

MyName = "SFFMerge"
MyVersion = "1.33"

VersionFile = "SFFVersion.txt"
VersionFolder = "_Admin"
DirectiveFile = "Directives.txt"
Log_Directory = "_Admin\"

Const WINDOW_HANDLE = 0
Const BIF_EDITBOX = &H10
Const BIF_NONEWFOLDER = &H0200
Const BIF_RETURNONLYFSDIRS = &H1
Const ForReading = 1
Const ForWriting = 2
Const ALLOWOVERWRITE = True
Const FORCEDELETE = True

intOptions = BIF_RETURNONLYFSDIRS + BIF_NONEWFOLDER + BIF_EDITBOX

Dim objFSO, files, folder, folders, CountCopied, CountTotal
Dim sm(15000,2)   ' files for special move
smcount = 0
Dim df(15000,2) ' files to delete
dfcount = 0

Set objFileSystem = CreateObject("Scripting.FileSystemObject")
Set objFSO = CreateObject("Scripting.FileSystemObject")

CountCopied = 0
CountTotal = 0
NeedVersion = "0000.0"
MakeVersion = "0000.0"
Separator = "	"                   'tab as separator
RBflag=False 						' flag to indicate rollback in progress.

Set objShell = CreateObject("Shell.Application")
Set fso = CreateObject("Scripting.FileSystemObject")

' Get input folder
If WScript.arguments.count > 0 Then 'Input folder dropped on script
	UpdateLocation = WScript.arguments.item(0)
Else
	strPrompt = "Select the folder where files to be merged are located."
	UpdateLocation = Browse4Folder(strPrompt, intOptions, baseupdates)
End If 

'Sanity check that UpdateLocation is a folder
'
If Not fso.folderexists(UpdateLocation) Then
	MsgBox "Not a Folder"
	WScript.quit
End If

' Get output (Library) folder
strPrompt = "Select the folder where your collection is located."
NewCollectionLocation = Browse4Folder(strPrompt, intOptions, baselibrary)

' Check for a folder 24 Declassified
If Not fso.folderexists(NewCollectionLocation + "\24 Declassified") Then
	MsgBox "Are you sure this is your Library?"
	WScript.quit
End If

' create default log/version directory if necessary
If Not fso.folderexists(NewCollectionLocation + "\_Admin") Then
	objFSO.CreateFolder(NewCollectionLocation + "\_Admin")
End If

' create default Merge Rollback directory if necessary
If Not fso.folderexists(NewCollectionLocation + "\_MergeRollback") Then
	objFSO.CreateFolder(NewCollectionLocation + "\_MergeRollback")
End If

' Check for a directives file

If fso.FileExists(UpdateLocation + "\" + Directivefile) Then 
	' Read File
	i = 0
	j = 0
	Set InFile = fso.OpenTextFile(UpdateLocation + "\" +  Directivefile, ForReading)
	' Process Directives
	Do Until InFile.AtEndOfStream
		s = inFile.ReadLine
		s = Trim(s)
		If Not s = "" Then
			s = Replace(s,"/","\") 'swap unix slash for windows one
			p = Split(s,separator)
			' Message Box
			If p(0) = "B" Then
				st = Replace(p(2), "<BR>", vbCr)
				If p(1) = 0 Then
					MsgBox st
				End If
				If p(1) = 1 Then
					MsgBox st
					WScript.quit
				End If
				If p(1) = 2 Then
					i = MsgBox(st, vbYesNo)
					If i = VbNo Then
						WScript.quit
					End If   
				End If
			End If	  
			' Delete and force delete
			If p(0) = "D" Or p(0) = "F" Then
				j = j + 1
				p(1) = Trim(p(1))
				If Not Left(p(1),1) = "\" Then
					p(1)="\" + p(1)
				End If
				If Right(p(1),1) = "\" Then
					p(1) = Left(p(1),Len(p(1)-1))
				End If
				If p(0) = "F" Then
					df(j,1) = 1
				Else
					df(j,1) = 0
				End If
				df(j,0) = p(1)
			End If
			' Special Move Instruction (allow for eg Star Trek files)
			If p(0) = "M" Then
				i = i + 1
				p(1) = Trim(p(1))
				p(2) = Trim(p(2))
				If Not Left(p(2),1) = "\" Then
					p(2) = "\" + p(2)
				End If
				If Right(p(2),1) = "\" Then 
					p(2) = Left(p(2),Len(p(2)-1))
				End If
				sm(i,0) = p(1)
				sm(i,1) = p(2)
			End If
			' Require Version
			If p(0) = "N" Then
				NeedVersion = p(1)
			End If
			' Rollback flag indicating rollback in progress
			If p(0) = "R" Then
				RBFlag = True
			End If
			' Make Version
			If p(0) = "V" Then
				MakeVersion = p(1)
			End If
			' check what version the script is
			If p(0) = "Y" Then 
				If (Not(p(1) = MyName)) Or p(2) > MyVersion Then
					MsgBox "You need " + p(1) + " Version " + p(2) + " to run this!" + vbCrLf + "Please download the latest copy from the forum"
					WScript.quit
				End If
			End If
		End If
	Loop
	smcount = i
	dfcount = j
	infile.Close
Else
	MsgBox ("No directives file")
	WScript.Quit	
End If

' Get current version
If fso.FileExists(NewCollectionLocation + "\" +  versionfolder + "\" + VersionFile) Then
	Set InFile = fso.OpenTextFile(NewCollectionLocation + "\" +  versionfolder + "\" + VersionFile, ForReading)
	originalsffVersion = infile.ReadLine
	infile.Close
Else
	originalsffVersion = "Unknown"
End If


' Validate Required Version
If Not NeedVersion = "0000.0" Then
	If Not versionfolder = "" Then
		If Not fso.FolderExists(NewCollectionLocation + "\" + versionfolder) Then
			MsgBox "Require Library Version " + NeedVersion + vbCrLf + "Can't identify your version" + vbCrLf + "No " + versionfolder + " Folder"
			WScript.quit
		End If
		If fso.FileExists(NewCollectionLocation + "\" + versionfolder + "\" + VersionFile) Then
			Set InFile = fso.OpenTextFile(NewCollectionLocation + "\" + versionfolder + "\" + VersionFile, ForReading)
		Else
			MsgBox "Require Library Version " + NeedVersion + vbCrLf + "Can't identify your version" + vbCrLf + "No " + versionfolder + "\" + versionfile + " File"
			WScript.quit
		End If
	Else
		If fso.FileExists(NewCollectionLocation + "\" +  VersionFile) Then
			Set InFile = fso.OpenTextFile(NewCollectionLocation + "\" +  VersionFile, ForReading)
		Else
			MsgBox "Require Library Version " + NeedVersion + vbCrLf + "Can't identify your version" + vbCrLf + "No " + versionfile + " File"
			WScript.quit
		End If
	End If
	CurrentVersion = infile.readline
	If Not NeedVersion = CurrentVersion Then
		MsgBox "Require Library Version " + NeedVersion + vbCrLf + "Current Version "+ CurrentVersion + vbCrLf + "Looks like you're not up to date!"
		WScript.quit
	End If
End If

If Not RBFlag Then
	' Check if version subfolder in _MergeRollback delete and recreate as necessary
	If fso.FolderExists(NewCollectionLocation+"\_MergeRollback\"+MakeVersion) Then
		fso.DeleteFolder NewCollectionLocation+"\_MergeRollback\"+MakeVersion,FORCEDELETE
	End If
	' Create rollback folder
	CreateFolder(NewCollectionLocation+"\_MergeRollback\"+MakeVersion)
	RB = NewCollectionLocation+"\_MergeRollback\"+MakeVersion
	' set up rollback directives file
	Set rbDirectives = FSO.CreateTextFile(NewCollectionLocation+"\_MergeRollback\"+MakeVersion+"\" + DirectiveFile)
	' some initial directives
	rbDirectives.WriteLine "R"+separator+"True"
	rbDirectives.WriteLine "V"+separator+currentversion
	rbDirectives.WriteLine "Y"+separator+MyName+separator+"1.30"
End If

' process delete directives
If dfcount > 0 Then
	For i = 1 To dfcount
		If fso.folderexists(NewCollectionLocation + df(i,0)) Then
			If df(i,1) = 0 Then
				' test if folder empty
				' normal delete folder
				Set folder = fso.GetFolder(NewCollectionLocation + df(i,0))
				If (folder.files.count + folder.subfolders.count) = 0 Then
					' no files or subfoders so delete folder
					' if not a rollback, copy the folder to the rollback directory
					If Not RBFlag Then fso.CopyFolder NewCollectionLocation + df(i,0),RB+df(i,0)
					fso.deletefolder NewCollectionLocation + df(i,0),FORCEDELETE
				Else
					MsgBox "Folder " +   + " not empty" + vbCrLf + "Please check and delete manually"
				End If
			Else
				' force delete
				' if not a rollback, copy the folder to the rollback directory
				If Not RBFlag Then 
					If Not fso.FolderExists(RB+df(i,0)) Then
					   CreateFolder(RB+df(i,0))
					End If   
					fso.CopyFolder NewCollectionLocation + df(i,0),RB+df(i,0)
				End if	
				fso.deletefolder NewCollectionLocation + df(i,0),FORCEDELETE
			End If
		Else
			If fso.fileexists(NewCollectionLocation + df(i,0)) Then
				' delete file
				If Not rbflag Then
					Set file = fso.GetFile(newcollectionlocation+df(i,0))
					' sort out "relative path" ie path relative to collection
					fname=Replace(file.Path,newcollectionlocation+"\","")
					' sort out relative folder
					fdr=Replace(fname,"\"+file.Name,"")
					' Create rollback folders as needed
					CreateFolder(rb+"\"+fdr) 
					' copy file to rollback folder
					fso.CopyFile NewCollectionLocation + df(i,0),RB+df(i,0)
				End If
				fso.deletefile NewCollectionLocation + df(i,0),FORCEDELETE ' FORCEDELETE allows us to remove RO files.
			End If
		End If
	Next
End If


' Move files/folders to the appropriate subdirectory
Set folder = fso.GetFolder(UpdateLocation)
Set folders = folder.subfolders
If folders.count > 0 Then
	' if there are any subfolders copy them across preserving any overwritten files 
	For Each folder In folders
		Movefolders(folder)
	Next		
End If

' Now move individual files
Set folder = fso.GetFolder(UpdateLocation)
Set files = folder.files
For Each file In files
	If Not file.name=DirectiveFile Then
		' Check if special move - move to alternate location
		sflag=False
		If smcount > 0 Then
			For i = 1 To smcount
				If file.name = sm(i,0) Then
					sflag = True
					smkr = i
				End If
			Next
		End If
		file.attributes = 0 'Clear file attributes - make sure nothing goea across with dodgy flags 
		If sflag Then
			' process special move
			If Not fso.folderexists(NewCollectionLocation + sm(smkr,1)) Then
				' if the folder doesn't exist, create it with a rollback directive
				CreateFolder1(NewCollectionLocation + sm(smkr,1))
			End If
			If Not rbflag Then
				'if not a rollback....
				If fso.FileExists(NewCollectionLocation + sm(smkr,1) + "\" + sm(smkr,0)) Then
					' if file exists create rollback folder (no directive!) if needed and copy file into rollback folder
					CreateFolder(rb+sm(smkr,1))
					fso.CopyFile NewCollectionLocation + sm(smkr,1) + "\" + sm(smkr,0),RB + sm(smkr,1) + "\" + sm(smkr,0)
				End If
			End If
			'copy new file to destination folder
			fso.copyfile updatelocation + "\" + sm(smkr,0),NewCollectionLocation + sm(smkr,1) + "\" + sm(smkr,0) , ALLOWOVERWRITE
			If Not RBflag Then
				' If not a rollback, create a D directive to delete the file on rollback
				tmpmk = Mid(sm(smkr,1),2,Len(sm(smkr,1)))
				rbdirectives.WriteLine("D"+separator+ tmpmk + "\" + sm(smkr,0))
			End If
		Else
			' move to author directory
			p = Split(file.name, " - ")
			author = p(0)
			If Not fso.folderexists(NewCollectionLocation + "\" + author) Then
				' create author folder with rollback directive if needed
				CreateFolder1(NewCollectionLocation + "\" + author)
			End If
			If Not rbflag Then
				' if not a rollback...
				If fso.FileExists(NewCollectionLocation + "\" + author + "\" + file.name) Then
					' if file already present create a rollback folder if needed and copy file to it
					CreateFolder(rb+"\"+author)
					fso.CopyFile NewCollectionLocation + "\" + author + "\" + file.name,RB + "\" + author + "\" + file.name
				End If
			End If
			' copy the new file to the author folder
			fso.copyfile UpdateLocation + "\" + file.name, NewCollectionLocation + "\" + author + "\" + file.name , ALLOWOVERWRITE
			' if not a rollback create a rollback D directive to delete the file on rollback
			If Not RBflag Then rbdirectives.WriteLine("D"+separator+author + "\" + file.name)
		End If
	End If
Next

' write new version file if required
If Not MakeVersion = "0000.0" Then 
	If Not versionfolder = "" Then
		If Not fso.folderexists(NewCollectionLocation + "\" + versionfolder) Then
			createfolder(NewCollectionLocation + "\" + versionfolder)
		End If
		Set OutFile = fso.CreateTextFile(NewCollectionLocation + "\" +  versionfolder + "\" + VersionFile, ForWriting)
	Else  
		Set OutFile = fso.CreateTextFile(NewCollectionLocation + "\" +  VersionFile, ForWriting)
	End If
	OutFile.writeline MakeVersion
	' if not a rollback, create a rollback N directive to give the version that rollback needs
	If Not RBFlag Then rbdirectives.WriteLine("N"+separator+MakeVersion)
End If

' Update History File - copied from SFFClean V2.01
ebookLocation = NewCollectionLocation
seperator = separator
Set folder = fso.GetFolder(updatelocation)
strInputFilePath = folder.name
DirectiveVersion = "0"

TStamp = TimeStamp()
If Not objFileSystem.FileExists(ebookLocation + "\" + Log_Directory + "Update_History.txt") Then
	Set objHistoryFile = objFileSystem.CreateTextFile(ebookLocation + "\" + Log_Directory + "Update_History.txt", True)
	objHistoryFile.WriteLine ("TimeStamp	SFF v	Update v	Update Filename")
	objHistoryFile.WriteLine ("------------------------------------------------------------------------")
	objHistoryFile.Close
End If
Set objHistoryFile = objFileSystem.OpenTextFile(ebookLocation + "\" + Log_Directory + "Update_History.txt", 8)
If Not rbflag Then
	' if not a rollback write history line
	objHistoryFile.WriteLine TStamp + seperator + originalsffVersion + seperator + MakeVersion + seperator + seperator + strInputFilePath
Else
	' otherwise write a rollback history line
	objHistoryFile.WriteLine TStamp + seperator + originalsffVersion + seperator + MakeVersion + seperator + seperator + "Merge Rollback "+strInputFilePath
End If
objHistoryFile.Close
Set objHistoryFile = Nothing

If Not rbflag Then
	'reverse rollback directives file to ensure directives processed in correct order
	' ie delete files before folders
	rbdirectives.Close
	Set objFile = FSO.OpenTextFile(NewCollectionLocation+"\_MergeRollback\"+MakeVersion+"\Directives.txt", ForReading)
	i = 0
	Do Until objFile.AtEndOfStream
		ReDim Preserve arrLines(i)
		arrLines(i) = objFile.ReadLine
		i = i + 1
	Loop
	objFile.Close
	
	Set objFile = FSO.OpenTextFile(NewCollectionLocation+"\_MergeRollback\"+MakeVersion+"\Directives.txt", ForWriting)
	For i = UBound(arrLines) To LBound(arrLines) Step -1
		objFile.WriteLine arrLines(i)
	Next
	objFile.Close
End If


MsgBox "Done!" 
WScript.quit
' End of main program


'=================== Subroutines ===========================

' Move subfolders directly
Sub Movefolders (f)
	' process each subfolder
	Set folds=f.subfolders
	For Each folder In folds
		' process any subfolders
		pth = folder.path
		pth = Replace(pth,UpdateLocation+"\","")
		'pth holds relative path
		If Not fso.FolderExists(NewCollectionLocation+"\"+pth) Then
			'create folder with rollback directive if needed
			CreateFolder1(NewcollectionLocation+"\"+pth)
		End If
		' and recursively process any sub-subfolders....
		Movefolders(folder)
	Next
	Set folds = f.files
	For Each file In folds
		' get relative file and folder
		fname=Replace(file.Path,UpdateLocation+"\","")
		fdr=Replace(fname,"\"+file.name,"")
		If Not RBFlag Then
			' if not a rollback
			If fso.FileExists(NewCollectionLocation + "\"+fname) Then
				' and file exists, create rollback folder if needed and copy old file to rollback
				CreateFolder(rb+"/"+fdr)
				fso.CopyFile NewcollectionLocation+"\"+fname,RB+"\"+fname
			End If
		End If
		' create folders with rollback as needed
		CreateFolder1(newcollectionlocation+"\"+fdr)
		' copy the new file
		fso.CopyFile file.Path,NewCollectionLocation+"\"+fname
		' and if not a rollback, create a rollback D directive
		If Not rbflag Then rbdirectives.WriteLine("D"+separator+fname)
	Next
End Sub	

'Recursive folder create, will create directories and parent folders as needed
Sub CreateFolder( strPath )
	On Error Resume Next
	If strPath <> "" Then 'Fixes endless recursion in some instances when at lowest directory
		If Not objFSO.FolderExists( objFSO.GetParentFolderName(strPath) ) Then Call CreateFolder( objFSO.GetParentFolderName(strPath) )
		If Not objFSO.FolderExists(strpath) Then
			objFSO.CreateFolder( strPath )
		End If
	End If 
End Sub

'Recursive folder create, will create directories and parents with rollback entries as needed
Sub CreateFolder1( strPath )
	On Error Resume Next
	If strPath <> "" Then 'Fixes endless recursion in some instances when at lowest directory
		If Not objFSO.FolderExists( objFSO.GetParentFolderName(strPath) ) Then Call CreateFolder1( objFSO.GetParentFolderName(strPath) )
		If Not fso.FolderExists(strpath) Then
			objFSO.CreateFolder( strPath )
			If Not RBFlag Then 
				pth=Replace(strpath,NewCollectionLocation+"\","")
				rbdirectives.WriteLine("D"+separator+pth)
			End If		
		End If
	End If 
End Sub

Function Browse4Folder(strPrompt, intOptions, strRoot)
	Dim objFolder, objFolderItem
	On Error Resume Next
	
	Set objFolder = objShell.BrowseForFolder(0, strPrompt, intOptions, strRoot)
	If (objFolder Is Nothing) Then
		WScript.Quit
	End If
	Set objFolderItem = objFolder.Self
	Browse4Folder = objFolderItem.Path
	Set objFolderItem = Nothing
	Set objFolder = Nothing
End Function

Function TimeStamp() 'return timestamp as YYYYMMDDHHMMSS
	Dim SplitTime, SplitDate, iDate, iTime
	
	myTime = Now
	myHour = CStr(Hour(myTime))
	myMinute = CStr(Minute(myTime))
	mySecond = CStr(Second(myTime))
	myDay = CStr(Day(mytime))
	myMonth = CStr(Month(mytime))
	myYear = CStr(Year(mytime))
	
	iDate = Right("0"+myYear,4)+ Right("0"+mymonth,2)+Right("0"+myDay,2)
	iTime = Right("0"+myHour,2)+ Right("0"+myminute,2)+Right("0"+mysecond,2)
	TimeStamp = iDate + iTime
End Function
